summaryrefslogtreecommitdiff
path: root/libavformat/rtpenc_xiph.c
diff options
context:
space:
mode:
authorMartin Storsjö <martin@martin.st>2015-02-26 00:00:39 +0200
committerMartin Storsjö <martin@martin.st>2015-02-28 22:54:31 +0200
commit4f6cd883f06f7893a2b60a41e7a4f8ae633dac2f (patch)
tree0a140108c19744a4399aa5d5244fd63d8b5e13bf /libavformat/rtpenc_xiph.c
parentbde2bba45c2f2df27a8534028bda09a6e7f835e2 (diff)
rtpenc: Don't set max_frames_per_packet based on the packet frame size or frame rate
Instead check the timestamps while muxing, to avoid buffering a too long timestamp range into one single packet. This makes the AMR and AAC packetization slightly less efficient, since we set a possibly unnecessarily high max_frames_per_packet. (These packetizers end up doing a memmove of the TOC bytes if sending a packet before max_frames_per_packet is achieved, and we end up setting max_frames_per_packet to a value that should be high enough for most uses.) All packetizers that use max_frames_per_packet now set it either to a default value, or to a value calculated based on other parameters, so none of them rely on the previous default setting. For iLBC, copy one frame at a time, to allow checking the timestamp range for each of them - basically doing potentially multiple loops to simplify the code instead of trying to calculate the number of frames to buffer while honoring s1->max_delay. This is in preparation for reducing the coupling between libavformat and libavcodec, by not having the muxers use the encoder field frame_size (which may not be available during e.g. stream copy). Signed-off-by: Martin Storsjö <martin@martin.st>
Diffstat (limited to 'libavformat/rtpenc_xiph.c')
-rw-r--r--libavformat/rtpenc_xiph.c7
1 files changed, 5 insertions, 2 deletions
diff --git a/libavformat/rtpenc_xiph.c b/libavformat/rtpenc_xiph.c
index 2167bdc99f..5b171c36a2 100644
--- a/libavformat/rtpenc_xiph.c
+++ b/libavformat/rtpenc_xiph.c
@@ -32,6 +32,7 @@
void ff_rtp_send_xiph(AVFormatContext *s1, const uint8_t *buff, int size)
{
RTPMuxContext *s = s1->priv_data;
+ AVStream *st = s1->streams[0];
int max_pkt_size, xdt, frag;
uint8_t *q;
@@ -77,8 +78,10 @@ void ff_rtp_send_xiph(AVFormatContext *s1, const uint8_t *buff, int size)
assert(s->num_frames <= s->max_frames_per_packet);
if (s->num_frames > 0 &&
(remaining < 0 ||
- s->num_frames == s->max_frames_per_packet)) {
- // send previous packets now; no room for new data
+ s->num_frames == s->max_frames_per_packet ||
+ av_compare_ts(s->cur_timestamp - s->timestamp, st->time_base,
+ s1->max_delay, AV_TIME_BASE_Q) >= 0)) {
+ // send previous packets now; no room for new data, or too much delay
ff_rtp_send_data(s1, s->buf, s->buf_ptr - s->buf, 0);
s->num_frames = 0;
}